/**
 * @file sht11.c
 * SHT11 Humidity Temperature Sensor
 *
 * Copyright (c) 2013, 
 * See end of file for terms of use.
 */

// #include <propeller.h>

#include "simpletools.h"
#include "sht11.h"

int DatPin;
int ClkPin;
sht11values_t  *avalues;


uint8_t cmmd;
long temp;
long fptempC;
long fptempF;
long humidity;
long fphumidity;
long fphumtrue;
long status;
long statusreg;


int Start(int DatP, int ClkP, sht11values_t *pValues) {
/*Parameters:
  DatP     : Data Pin of SHT11
  ClkP     : Clock Pin of SHT11
  aValues  : Address of data struct to place measurement result values
              long rawhum
              long rawtemp
              long rhlin
              long rhtrue
              long tempC
              long tempF

*/
/*Copy the parameters to globle variables */
  DatPin = DatP;
  ClkPin = ClkP;
  avalues = pValues;
  ResetSeq();
//  set_direction(scopepin,1);
#ifdef DEBUGSCOPE
  set_direction(scopepin,1);  // Set it as an output to trigger scope
#endif  
  return(0);
 }
 
 
int SetIdle() {
  /*Set pins to idle state */

  high(DatPin);   //Pin High
  low(ClkPin);   //Pin Low
  
  return(0);
}


int StartSeq() {
  /*Send an SHT11 Command Start Sequence*/

  SetIdle();             //Set idle state
  //Raise the clock pin while data pin is high
  high(ClkPin);
  //Lower the data pin while the clock pin is high
  low(DatPin);
  //Pulse the clock pin
  low(ClkPin);
  high(ClkPin);
  //Raise data pin while clock pin is high
  high(DatPin);
  //Return clock pin low to compelte sequence and return to idle state
  low(ClkPin);
#ifdef TRIGSTARTSEQ
  /*Scope Trigger*/
  high(scopepin);
  low(scopepin);
#endif  
   
  return(0);
  
}
 

int ResetSeq() {
  /*Send an SHT11 Reset Sequence*/

  SetIdle();            //Set idle state
  //Send 9 clocks
  for(int i=0;i<9;i++) {
    high(ClkPin);
    low(ClkPin);
  }

#ifdef TRIGRESETSEQ
  /*Scope Trigger*/
  high(scopepin);
  low(scopepin);
#endif  
  //We need to give it at least 11ms to wake up, do 100ms
  pause(100);   // waitcnt(clkfreq/10 + cnt)
  //Send the Start Sequence

  set_direction(DatPin,0); // Set it as input so Measure cog can control it because ResetSeq is only called
                           // from this obj but the Measure Method is in it//s own cog and needs to control
                          //the pins. Remember anyone who sets the pin high blocks all others from setting it
                          //low ie logic or a 1 wins
  return(0);              
}                

int SendCommand(uint8_t Cmd) { 
  /*Send a command uint8_t to the SHT11, if uint8_t is acknowledged on last clock return True*/
  /*Leaves data pin as an input for following read of temp or humidity*/
  /*Data pin is an input on exit, so ReadValue can watch for convert complete signal*/
  
int idx;
unsigned int ACQ;
uint8_t tmp;


  StartSeq();
  idx = 7;
//  Repeat idx from 7 to 0
  while(idx >= 0) {
    tmp = (Cmd >> idx) & 0x01;  // Set output to bit 0
    set_output(DatPin, tmp);
    high(ClkPin);
    low(ClkPin);
    idx--;
  }

  //Switch Data Pin to an input and check for a low ACQ
  set_direction(DatPin, 0);   //Set it low for an input
  //Set the Clock bit high
  high(ClkPin);
  //Read the Data Pin for ACQ of command being low
  ACQ = input(DatPin);
  //Clock back low
  low(ClkPin);
  
#ifdef TRIGCOMMAND
  /*Scope Trigger*/
  high(scopepin);
  low(scopepin);
#endif  
  return ( ACQ == 0);
}

long ReadValue() {
  /*Read Temp or Humidity after SendCommand(Cmd)*/
  /*Does not ACQ the last uint8_t causing CRC to not be sent by SHT11*/
long Value;
unsigned int mask;
unsigned int pinvalue;  
  
  //Wait for the SHT11 to drop the data line
//  waitpeq(|< DatPin,|< DatPin,0)   // Wait for DatPin to be high
  mask = 1 << DatPin;
  waitpeq(0 << DatPin,mask);  // Wait for DatPin to go low
  Value = 0;
  //Read 8 data values for the MSB of Value
  //Repeat 8
  for(int j=0;j<8;j++) {
    pinvalue = input(DatPin);
    Value = (Value << 1) | pinvalue;  //Shift one bit over and or in the next bit
    //Set clock pin high then low
    high(ClkPin);
    low(ClkPin);
  }  
  //We did the eighth clock low so switch to data line as an output and ACQ by setting data low
  low(DatPin);
  //Cycle the clock
  high(ClkPin);
  low(ClkPin);
  //Switch back to data pin as an input to read LSB of Value
  set_direction(DatPin, 0);
  //Repeat 8
  for(int i=0;i<8;i++) {
    pinvalue = input(DatPin);
    Value = (Value << 1) | pinvalue;  //Shift one bit over and or in the next bit
    //Set clock pin high then low
    high(ClkPin);
    low(ClkPin);
  }  
  //Clock is low after eight clocks, set data pin to output and set it high to not ACQ
  //this will end transmission and ignore CRC
  high(DatPin);
  //Cycle clock to allow SHT11 to read or NAQ
  high(ClkPin);  //Set ClkPin high
  low(ClkPin);   //Set ClkPin low

  //Set the data pin to an input to release control of it
  set_direction(DatPin, 0);
  
  
#ifdef TRIGREAD
  /*Scope Trigger*/
  high(scopepin);
  low(scopepin);
#endif  
  return ( Value );
}

long ReadStatusReg() {
  /*Read the SHT11 Status Register*/
long Value;
unsigned int pinvalue;
long stat;  
unsigned int mask;

 
   stat = SendCommand(KREADSTATUS);

  //Wait for the SHT11 to drop the data line
//  waitpeq(|< DatPin,|< DatPin,0)   // Wait for DatPin to be high
  mask = 1 << DatPin;
  waitpeq(0 << DatPin,mask);  // Wait for DatPin to go low
  Value = 0;
  //Repeat 8
  for(int i=0;i<8;i++){
    pinvalue = input(DatPin);
    Value = (Value << 1) | pinvalue;   //Shift one bit over and or in the next bit
    //Set clock pin high
    high(ClkPin);
    low(ClkPin);
  }
  //Clock is low after eight clocks, set data pin to output and set it high to not ACQ
  //this will end transmission and ignore CRC
  high(DatPin);
  //Cycle clock to allow SHT11 to read or NAQ
  high(ClkPin);  //Set ClkPin high
  low(ClkPin);   //Set ClkPin low

  //Set the data pin to an input to release control of it
  set_direction(DatPin, 0);
#ifdef TRIGREADSTAUS
  /*Scope Trigger*/
  high(scopepin);
  low(scopepin);
#endif  
  return(Value);
}

long RawToTempC(long raw) {
long tmpC;

/* tmpC = .01 * raw - 39.60   @ 3.3 Volts*/

  tmpC = Kd2C_0_16 * raw;   // 16:0 raw counts * 0:16 constant ==> 16:16 answer
  /*Add and Sub in fixed point needs same format which we have*/
  tmpC = tmpC - Kd1C_16_16;   // 16:16 + 16:16 ==> 16:16
  /*Remove the shift and round*/
  tmpC = tmpC/ 65536L;
  
  return(tmpC);
}

long RawToTempF(long raw) {
long tmpF;

/* tmpF = .018 * raw - 39.32   @ 3.3 Volts*/

  tmpF = Kd2F_0_16 * raw;   // 16:0 raw counts * 0:16 constant ==> 16:16 answer
  /*Add and Sub in fixed point needs same format which we have*/
  tmpF = tmpF - Kd1F_16_16;   // 16:16 + 16:16 ==> 16:16
  /*Remove the shift and round*/
  tmpF = tmpF/ 65536L;
  
  return(tmpF);
}

long ConvertC2F(long degC){
long tmpF;

/* DegF = 9/5 DegC + 32 */
/* 9/5 = 1.8
  DegF = 1 * DegC + 0.8 * DegC + 32
  0.8 * 65536 = 52428 */
  
  /*Compute 10 * temp F from 10 * temp C*/
  tmpF = degC + (52428 * degC)/ 65536L + 320L;  // We have 10 time DegC so 320 not 32

  /*Try the same calculation using Propellers ** operator which does the divide
   We need 0.8 as a 32 bit fraction for this 0.8 = 3,435,973,836 */
//   tmpF = degC + 3435973836 ** DegC + 320 // Wrong because we get a negitive num for ** operand

/* tmpF = 90 * 256 * degC          //8:8 * 16:0 ==> 24:8
 tmpF = tmpF / (50 * 256 * degC) //24:8 / 24:8 ==> Shift taken out 0:0
*/
  return(tmpF);
}
 

long RawToRHlin(long raw) {
long rhl;

   /*Temp conversion constants from Sensirion data sheet*/
   /*Convert 10 Times to get 0.1 degrees*/


  /*Tracy Allens formula*/
//  rhl = (4 * raw) +(214748365 ** raw) - (1202591 ** (raw * raw)) -400;

  /*Use a 24bit frac part for better acuracy of 2.8e-5*/
  rhl = Kc3_0_24 * raw;       // 0:24 * 16:0 ==> 16:24 could need 40 bits if raw large enough
  rhl = rhl / 256;            // Now 16:16  24bits shifted 8 leaves 16 bits of fraction

//  rhl = Kc3_0_16 * raw      // 16:0 times 0:16 ++> 16:16 only 16bit frac of 2.8e-5 ==> 1.8 ie 1 or 2
  rhl = -rhl;                //c3 is actually a negitive number
  /*Now have a 16:16 format */
  rhl = rhl + Kc2_0_16;     //Have a 16:16 result
  rhl = rhl * raw;            // 16:16 times 16:0 ==> 32:16
//  rhl = rhl / 65536;          //32:16 div 16:16 ==>16:0  65536 is 1 over 16 bits or 16:16
  rhl = rhl - Kc1_16_16;       //c1 is actually a minus number
  rhl = rhl /65536L;
  
  return(rhl);
}

int Measure() {
long rhtru;
long stat;
long humraw;
long humlin;
long tmpC;
long traw;

/*Take a measurement of Temp and RH Humidity Values*/

/* Do the complete measurement of Temp ==> RHlin ==> Rhtrue */
/* Computing 10 times Values*/


    /*Get the raw humidity counts and convert th RHlin*/
    stat = SendCommand(KREADHUM);
    humraw = ReadValue();
    humlin = RawToRHlin(humraw);
    //waitcnt(clkfreq/1000 + cnt)
    pause(1);
    /*Get the raw temp counts and convert to Deg C*/
    stat = SendCommand(KREADTEMP);
    traw = ReadValue();
    tmpC = RawToTempC(traw);
    //waitcnt(clkfreq/1000 + cnt)
    pause(1);

    /*Correct humlin for a temp different than 25 Deg C*/
    rhtru = Kt2_0_16 * humraw;    // 0:16 * 16:0 ==> 16:16  t2 * raw counts
    rhtru = rhtru  + Kt1_0_16;     // 16:16 + 0:16  ==> 16:16  t1 + t2 * raw
    /*Had to divide temp dif by 10 because without it the result of the multiply became
     a false negitive number. Without the div by 10 we are actually calculating 100 times
     because we have 10 times the temp dif and 10 times the constants.
     Another way to do it would be to only do the constants as is without multipling them by 10.
     That is what Tracy did*/
    rhtru = rhtru  * ( tmpC - 250L)/10L; // 16:16 * 16:0  ==> 32:16
    rhtru = rhtru  / 65536L;         // 16:0
    rhtru = rhtru + humlin;       // ==> 10 times RH true
    
    /*Update the values*/
    avalues->rawhum =  humraw;
    avalues->rawtemp = traw;
    avalues->rhlin   = humlin;
    avalues->rhtrue  = rhtru;
    avalues->tempC  = tmpC;
    avalues->tempF   = RawToTempF(traw);
    
    return(0);
}
    



/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                             
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/

